;;;                                                                    ;
;;;  atableG2G.LSP                                                     ;
;;;                                                                    ;
;;;  Copyright 2005 by Alexander Shchetinin. All Rights Reserved.      ;
;;;                                                                    ;
;;;  You are hereby granted permission to use, copy and modify this    ;
;;;  software without charge, provided you do so exclusively for       ;
;;;  your own use or for use by others in your organization in the     ;
;;;  performance of their normal duties, and provided further that     ;
;;;  the above copyright notice appears in all copies and both that    ;
;;;  copyright notice and the limited warranty and restricted rights   ;
;;;  notice below appear in all supporting documentation.              ;
;;;                                                                    ;
;;;  Incorporation of any part of this software into other software,   ;
;;;  except when such incorporation is exclusively for your own use    ;
;;;  or for use by others in your organization in the performance of   ;
;;;  their normal duties, is prohibited without the prior written      ;
;;;  consent of author.                                                ;
;;;                                                                    ;
;;;  Copying, modification and distribution of this software or any    ;
;;;  part thereof in any form except as expressly provided herein is   ;
;;;  prohibited without the prior written consent of author.           ;
;;;                                                                    ;
;;;  AUTHOR PROVIDES THIS SOFTWARE "AS IS" AND WITH ALL FAULTS.        ;
;;;  AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF             ;
;;;  MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTHOR,         ;
;;;  DOES NOT WARRANT THAT THE OPERATION OF THE SOFTWARE               ;
;;;  WILL BE UNINTERRUPTED OR ERROR FREE.                              ;
;;;                                                                    ;
;;;--------------------------------------------------------------------;
;;;       Function:  atableG2G                                         ;
;;;                                                                    ;
;;;    Description:  This function is copy pseudo-table text from      ;
;;;                  drawing to active grid in ATable editor.          ;
;;;                                                                    ;
;;;                                                                    ;
;;;                  Required Application: ATable for AutoCAD          ;
;;;                                                                    ;
;;;      Arguments:  None                                              ;
;;;                                                                    ;
;;; Returned Value:  None	                                       ;
;;;                                                                    ;
;;;          Usage:  Load atableG2G.lsp in AutoCAD by command Appload. ;
;;;                  Run ATable for AutoCAD. Display ATable editor     ;
;;;                  Enter option Add-ons in command line. Select G2G  ;
;;;                  option and follow instructions in command line.   ;
;;;                                                                    ;
;;; Specially thanks to Peter Loskutov for convert "engine".           ;
;;;                                                                    ;
;;;--------------------------------------------------------------------;

;You can localize this application. Only change text strings below.
(setq atableG2Glocal
  (list 
    (cons 20 "Alxd ATable is closed or unavailable!")
    (cons 30 "Active sheet of Alxd ATable is closed or unavailable!")
    (cons 69 "eXit")
    (cons 70 "\nSpecify first corner or [eXit]: ")
    (cons 71 "\nSpecify opposite corner: ")
    (cons 120 "nothing selected...")
    (cons 121 "\nPlease wait...")
    (cons 122 "done.")
  )
)

(defun getLocal ( slocal nlocal / )
  (cdr (assoc nlocal slocal))
)

(defun getActiveSheet(atObject / atEditor atSS atActiveSheet wi)
   (setq
      atEditor (vlax-get-property atObject 'AlxdEditor)
      atSS (vlax-get-property atEditor 'AlxdSpreadSheets)
      wi (vlax-get-property atSS 'Active)
      atActiveSheet (vlax-get-property atSS 'Items wi)
   )
  (vlax-release-object atSS)
  (vlax-release-object atEditor)
  atActiveSheet
)

(defun list2grid ( atObject pt1 pt2 / textlist atActiveSheet atCols atRows atCells atCell wi wj li lj)
  (princ (getlocal atableG2Glocal 121))

  ;is not my function
  ;great thanks to Peter Loskutov
  (setq textlist (pl:convert-to-table (list pt1 pt2)))

  (if (= textlist nil)
    (princ (getlocal atableG2Glocal 120))
    (progn
      (setq 
        atActiveSheet (getActiveSheet atObject)
        atCols (vlax-get-property atActiveSheet 'Columns)
        atRows (vlax-get-property atActiveSheet 'Rows)
        atCells (vlax-get-property atActiveSheet 'Cells)
      )
      (vlax-put-property atActiveSheet 'EditorMode :vlax-false)

      (vlax-put-property atRows 'Count (length textlist))

      (setq wj 0)
      (foreach li textlist

        (setq wi 0)
        (foreach lj li
          (if (> (length li) (vlax-get-property atCols 'Count))
            (vlax-put-property atCols 'Count (length li))
          )

          (setq atCell (vlax-get-property atCells 'Items wi wj))
          (vlax-put-property atCell 'Contain lj)
          (vlax-release-object atCell)

          (setq wi (1+ wi))
        );foreach lj

        (setq wj (1+ wj))
      );foreach li
 
      (vlax-invoke-method atActiveSheet 'Recalculate)
      (vlax-invoke-method atActiveSheet 'RedrawBlockDefinition 0 0)

      (vlax-release-object atCells)
      (vlax-release-object atRows)
      (vlax-release-object atCols)
      (vlax-release-object atActiveSheet)
    )
  )

  (princ (getlocal atableG2Glocal 122))  
)

(defun interfaceG2G ( atObject / inistr cmdstr temp xx xx2)
  (setq temp T)

  (setq
    cmdstr (getlocal atableG2Glocal 70)
    inistr (getlocal atableG2Glocal 69)
  )

  (while temp
    (initget 128 inistr)
    (setq xx (getpoint cmdstr))

    (cond
      ((= 'LIST (type xx))
        (setq xx2 (getcorner xx (getlocal atableG2Glocal 71)))
        (if (= 'LIST (type xx2))
          (list2grid atObject xx xx2)
          (princ (getlocal atableG2Glocal 120))
        )
      )
      ((or (= xx nil) (= xx (getlocal atableG2Glocal 69))) (setq temp nil))
    );cond

  );while
)

(defun c:atableG2G( / sstext atObject)

  ;get alxd atable object here
  (setq atObject (vlax-get-object "AlxdGrid.AlxdApplication"))

  ;check value
  (if (= atObject nil)
    (progn
      (alert (getlocal atableX2Glocal 20))
      (exit)
    )
  )

;debug string
;  (vlax-dump-object atObject)

  (vl-load-com)
  (vl-catch-all-apply 'interfaceG2G (list atObject))
  (vlax-release-object atObject) 
 
  (princ)
)

;-----------------------------------
;
;COOL code below from Peter Loskutov
;
;-----------------------------------

(defun pl:convert-to-table (pnts)
  (if pnts
    (mapcar (function (lambda (a) (mapcar (function (lambda (b) (vl-string-trim " " b))) a)))
            (pl:get-tbl-data1 (pl:get-tbl-ents1 pnts)) ;_   
    ) ;_       
  )
)


(defun pl:get-tbl-ents1 (_box) ;_     
  (setq _box (list (list (min (caar _box) (caadr _box)) (max (cadar _box) (cadadr _box)))
                   (list (max (caar _box) (caadr _box)) (min (cadar _box) (cadadr _box)))
             )
  )
  (list _box
        (ssget "_C" (car _box) (cadr _box) '((0 . "LINE")))
        (ssget "_C" (car _box) (cadr _box) '((0 . "LWPOLYLINE")))
        (ssget "_C" (car _box) (cadr _box) '((0 . "TEXT")))
  )
)

(defun pl:get-tbl-data1 (_sel / _sel _texts _lhrzn _lines _lvert _lwpl _modcol _modrow _mtx)
  (if (setq _texts (last _sel))
    (progn
      (setq _lines (cadr _sel)
            _lwpl  (caddr _sel)
            _sel   (car _sel)
      ) ;_ end of setq
      (if _lines
        (setq _lines (mapcar (function pl:extr-pnt-from-line) (pl:entlst-from-ss _lines)))
      ) ;_ end of if
      (if _lwpl
        (setq _lwpl (apply (function append)
                           (mapcar (function pl:lwpl-to-segments) (pl:entlst-from-ss _lwpl))
                    )
        ) ;_ end of setq
      ) ;_ end of if
      (if
        (and
          (setq _lines (append _lines _lwpl))
          (setq _lines (vl-remove-if-not
                         (function (lambda (x) (pl:is-point-in-bbox (pl:get-cen-pnts-2d x) _sel)))
                         _lines
                       ) ;_ end of vl-remove-if-not
          ) ;_ end of setq
          (setq _lines (mapcar (function (lambda (x) (pl:near-orto x 3))) _lines))
          (> (length (setq
                       _lvert (mapcar
                                (function cdr)
                                (vl-remove-if (function (lambda (x) (or (not x) (= (car x) 0)))) _lines)
                              ) ;_ end of mapcar
                     ) ;_ end of setq
             ) ;_ end of length
             1
          ) ;_ end of >
          (> (length (setq
                       _lhrzn (mapcar
                                (function cdr)
                                (vl-remove-if (function (lambda (x) (or (not x) (= (car x) 1)))) _lines)
                              ) ;_ end of mapcar
                     ) ;_ end of setq
             ) ;_ end of length
             1
          ) ;_ end of >
        ) ;_ end of and
         (progn (setq _modcol (pl:get-len-perc _lvert 1.0)
                      _modrow (* 0.5
                                 (apply (function min)
                                        (mapcar (function vla-get-height)
                                                (setq _texts (mapcar (function vlax-ename->vla-object)
                                                                     (pl:entlst-from-ss _texts)
                                                             )
                                                ) ;_ end of setq
                                        ) ;_ end of mapcar
                                 ) ;_ end of apply
                              ) ;_ end of *
                      _lvert  (pl:clr-near-doub (pl:sort (function <) _lvert) _modcol)
                      _lhrzn  (pl:sort (function >) (pl:clr-near-doub (pl:sort (function <) _lhrzn) _modrow))
                      _texts  (mapcar (function list)
                                      (mapcar (function pl:get-cen-pnts) (mapcar (function pl:get-bbox) _texts))
                                      _texts
                              )
                      _mtx    (pl:mk-arr-from-lns _lhrzn _lvert)
                      _mtx    (mapcar (function (lambda (a b) (mapcar (function list) a b)))
                                      _mtx
                                      (mapcar (function cdr) (cdr _mtx))
                              ) ;_ end of mapcar
                      _mtx    (mapcar
                                (function
                                  (lambda (a)
                                    (mapcar
                                      (function (lambda (b)
                                                  (pl:txts-conc1
                                                    (vl-remove-if-not
                                                      (function (lambda (c) (pl:is-point-in-bbox (car c) b)))
                                                      _texts
                                                    ) ;_ end of vl-remove-if-not
                                                  ) ;_ end of cadar
                                                ) ;_ end of lambda
                                      ) ;_ end of function
                                      a
                                    ) ;_ end of mapcar
                                  ) ;_ end of lambda
                                ) ;_ end of function
                                _mtx
                              ) ;_ end of mapcar
                ) ;_ end of setq
                (mapcar (function (lambda (b)
                                    (mapcar (function (lambda (c)
                                                        (cond (c)
                                                              (t "")
                                                        ) ;_ end of cond
                                                      ) ;_ end of lambda
                                            ) ;_ end of function
                                            b
                                    ) ;_ end of mapcar
                                  ) ;_ end of lambda
                        ) ;_ end of function
;;;                   (vl-remove-if-not (function (lambda (a) (apply (function or) a))) _mtx)
                        _mtx ;_     ,  -      .
                ) ;_ end of mapcar
         ) ;_ end of progn
      ) ;_ end of if
    ) ;_ end of progn
  ) ;_ end of if
) ;_ end of defun

(defun pl:txts-conc1 (i-lst) ;_       
  (cond ((not i-lst) nil)
        ((= (length i-lst) 1) (vla-get-textstring (cadar i-lst)))
        (t
         (apply (function strcat)
                (mapcar (function (lambda (c / tmp)
                                    (setq tmp (vl-string-trim " " (vla-get-textstring (cadr c))))
                                    (if (= (last (vl-string->list tmp)) 45)
                                      (vl-string-right-trim "-" tmp)
                                      (strcat tmp " ")
                                    ) ;_ end of if
                                  ) ;_ end of lambda
                        ) ;_ end of function
                        (pl:sort (function (lambda (a b / tmp)
                                             (setq tmp (angle (car a) (car b)))
                                             (or (< 0 tmp 0.52359878) (< 4.1887902 tmp 6.2831853))
                                           ) ;_ end of lambda
                                 ) ;_ end of function
                                 i-lst
                        ) ;_ end of pl:sort
                ) ;_ end of mapcar
         ) ;_ end of apply
        )
  ) ;_ end of cond
) ;_ end of defun

(defun pl:mk-arr-from-lns (col row) ;_     
  (mapcar (function (lambda (a) (mapcar (function (lambda (b) (list b a))) row))) col)
) ;_ end of defun

(defun pl:clr-near-doub (ilst mod / el tmp)
  (if ilst
    (progn (setq el  (car ilst)
                 tmp (pl:clr-near-doub (cdr ilst) mod)
           ) ;_ end of setq
           (if (and tmp (> el (- (car tmp) mod)))
             tmp
             (cons el tmp)
           ) ;_ end of if
    ) ;_ end of progn
  ) ;_ end of if
) ;_ end of defun

(defun pl:get-len-perc (lst perc) ;_         
  (* (abs (- (apply (function max) lst) (apply (function min) lst))) 0.01 perc)
)

(defun pl:extr-pnt-from-line (_line / _p1 _p2) ;_     
  (setq _line (entget _line)
        _p1   (cdr (assoc 10 _line))
        _p2   (cdr (assoc 11 _line))
  ) ;_ end of setq
  (list (list (car _p1) (cadr _p1)) (list (car _p2) (cadr _p2)))
) ;_ end of defun

(defun pl:extr-pnt-from-lwline (_dxf) ;_      
  (mapcar (function cdr) (vl-remove-if-not (function (lambda (x) (= 10 (car x)))) _dxf))
) ;_ end of defun

(defun pl:lwpl-to-segments (_lwline / _vert) ;_      
  (setq _lwline (entget _lwline)
        _vert   (pl:extr-pnt-from-lwline _lwline)
  ) ;_ end of setq
  (mapcar (function list)
          (if (zerop (logand 1 (cdr (assoc 70 _lwline))))
            (cdr _vert)
            (cons (last _vert) _vert)
          ) ;_ end of if
          _vert
  ) ;_ end of mapcar
) ;_ end of defun

(defun pl:near-orto (_lstpnt _delta / _ang _dir) ;_         
  (setq _ang   (rem (apply (function angle) _lstpnt) pi)
        _delta (rem (/ (* pi _delta) 180) (* pi 2))
        _dir   (cond ((>= (+ (/ pi 2) _delta) _ang (- (/ pi 2) _delta)) 1)
                     ((or (>= _delta _ang 0) (>= pi _ang (- pi _delta))) 0)
                     (t nil)
               ) ;_ end of cond
  ) ;_ end of setq
  (if _dir
    (cons _dir
          (/ (apply (function +)
                    (mapcar (if (= _dir 0)
                              (function cadr)
                              (function car)
                            ) ;_ end of if
                            _lstpnt
                    ) ;_ end of mapcar
             ) ;_ end of apply
             2
          ) ;_ end of /
    ) ;_ end of cons
  ) ;_ end of if
) ;_ end of defun

(defun pl:sort (func lst) ;_   
  (mapcar (function (lambda (x) (nth x lst))) (vl-sort-i lst func))
)

(defun pl:entlst-from-ss (ss) ;_      
  (vl-remove-if (function listp) (mapcar (function cadr) (ssnamex ss)))
)

(defun pl:get-bbox (obj / minpoint maxpoint) ;_    
  (vla-getboundingbox obj 'minpoint 'maxpoint)
  (mapcar (function vlax-safearray->list) (list minpoint maxpoint))
) ;_ end of defun

(defun pl:is-point-in-bbox (point bbox) ;_      
  (apply (function and)
         (mapcar (function (lambda (x y) (<= (apply (function min) x) y (apply (function max) x))))
                 (apply (function mapcar) (cons (function list) bbox))
                 point
         ) ;_ end of mapcar
  ) ;_ end of apply
) ;_ end of defun

(defun pl:get-cen-pnts (pntlst / len) ;_    
  (setq len (length pntlst))
  (list (/ (apply (function +) (mapcar (function car) pntlst)) len)
        (/ (apply (function +) (mapcar (function cadr) pntlst)) len)
        (/ (apply (function +) (mapcar (function caddr) pntlst)) len)
  ) ;_ end of list
) ;_ end of defun

(defun pl:get-cen-pnts-2d (pntlst / len) ;_     
  (setq len (length pntlst))
  (list (/ (apply (function +) (mapcar (function car) pntlst)) len)
        (/ (apply (function +) (mapcar (function cadr) pntlst)) len)
  )
) ;_ end of defun

(princ "\natableG2G v1.1 (addon for ATable) - input pseudo-table from drawing to tables")
(princ)
